'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.lockyOn = exports.lockyGroup = undefined;

var _utils = require('./utils');

var _handleScroll = require('./handleScroll');

var _defaultEvents = require('./defaultEvents');

var _isInside = require('./isInside');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var lockyGroup = exports.lockyGroup = function lockyGroup(selector, group) {
  [].concat(_toConsumableArray(document.querySelectorAll(selector))).forEach(function (node) {
    return node.setAttribute(_isInside.LOCKY_GROUP, group);
  });
};

var lockyOn = exports.lockyOn = function lockyOn(selector) {
  var settings = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var ref = typeof selector === 'string' ? document.querySelector(selector) : selector;

  if (!ref) {
    throw new Error('locky: selector', selector, 'is not matching any element.');
  }

  var touchStart = 0;

  var isEventInLock = function isEventInLock(event) {
    return ref && (0, _isInside.isInside)(ref, event.target);
  };

  var getEventHandlers = function getEventHandlers() {
    var noDefault = settings.noDefault,
        events = settings.events;

    return Object.assign({}, noDefault ? {} : _defaultEvents.EVENTS, events || {});
  };

  var getEventHandler = function getEventHandler(eventName, option) {
    var handler = (0, _utils.getHandler)(eventName, option, settings.onEscape);
    if (handler) {
      return function (event) {
        if (!isEventInLock(event) && !(0, _isInside.shouldIgnoreEvent)(event.target)) {
          handler(event);
        }
      };
    }
    return null;
  };

  var scrollWheel = function scrollWheel(event) {
    return (0, _handleScroll.handleScroll)(ref, event, event.deltaY, settings.preventOnly);
  };
  var scrollTouchStart = function scrollTouchStart(event) {
    touchStart = (0, _handleScroll.getTouchY)(event);
  };
  var scrollTouchMove = function scrollTouchMove(event) {
    return (0, _handleScroll.handleScroll)(ref, event, touchStart - (0, _handleScroll.getTouchY)(event), settings.preventOnly);
  };

  var handlers = getEventHandlers();
  var documentEvents = Object.keys(handlers).map(function (event) {
    return (0, _utils.addEvent)(document, event, getEventHandler(event, handlers[event]), true);
  }).filter(Boolean);

  var nodeEvents = [];
  if (handlers.scroll) {
    nodeEvents.push.apply(nodeEvents, [(0, _utils.addEvent)(ref, 'wheel', scrollWheel, true), (0, _utils.addEvent)(ref, 'touchstart', scrollTouchStart, true), (0, _utils.addEvent)(ref, 'touchmove', scrollTouchMove, true)]);
  }

  if (settings.group) {
    lockyGroup(selector, settings.group);
  }

  return function () {
    documentEvents.forEach(_utils.removeEvent);
    nodeEvents.forEach(_utils.removeEvent);
  };
};