'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
var preventAll = exports.preventAll = function preventAll(event) {
  event.preventDefault();
  event.stopPropagation();
  event.stopImmediatePropagation();
};

var preventDefault = exports.preventDefault = function preventDefault(event) {
  event.preventDefault();
};

var report = function report(callback) {
  return function (event) {
    preventAll(event);
    if (callback) {
      callback(event);
    }
  };
};

var reportOnly = function reportOnly(callback) {
  return function (event) {
    if (callback) {
      callback(event);
    }
  };
};

var getHandler = exports.getHandler = function getHandler(event, option, callback) {
  if (!option) {
    return null;
  }

  if (option === 'no-default') {
    return preventDefault;
  }

  if (option === 'report') {
    return report(callback);
  }

  if (option === 'report-only') {
    return reportOnly(callback);
  }

  return preventAll;
};

var addEvent = exports.addEvent = function addEvent(target, event, handler, capture) {
  return handler && {
    target: target,
    event: event,
    handler: (target.addEventListener(event, handler, capture), handler),
    capture: capture
  };
};

var removeEvent = exports.removeEvent = function removeEvent(_ref) {
  var target = _ref.target,
      event = _ref.event,
      handler = _ref.handler,
      capture = _ref.capture;
  return target.removeEventListener(event, handler, capture);
};